//
//  PreallocTableViewController.m
//  Week3_ControllerExample
//
//  Created by Jason Fieldman on 1/25/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "PreallocTableViewController.h"

@interface PreallocTableViewController (Priv)
- (void) allocateTableViewCells;
@end



@implementation PreallocTableViewController


- (id) init {
	if (self = [super init]) {
		self.title = @"Prealloc Tab.";
		self.tabBarItem = [[[UITabBarItem alloc] initWithTitle:@"Prealloc Tab." image:[UIImage imageNamed:@"tabbar_icon.png"] tag:0] autorelease];
		
		self.view = [[[UIView alloc] initWithFrame:CGRectMake(0, 0, 320, 480)] autorelease];
		self.view.autoresizesSubviews = YES;
		self.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
		
		/* By setting the autoresizingMask, we help ensure that our table will be the proper size even
		   if we're shrunk inside multiple layers of navigation.
		 
		   We also need to make sure that our self.view.autoresizesSubviews is YES, so that this resizing will
		   take place.
		 */
		
		myTableView = [[[UITableView alloc] initWithFrame:CGRectMake(0, 0, 320, 480)] autorelease];
		myTableView.delegate = self;
		myTableView.dataSource = self;
		myTableView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
		[self.view addSubview:myTableView];
		
		/* We call our message to allocate cells ahead of time.  Notice that allocateTableViewCells: was 
		   declared in our category declaration above.  This is the same thing as a function prototype.
		 
		   By preallocating our cells, we eliminate the effects of sluggish cell loading as the table scrolls (since
		   the cells are pregenerated).  However, we must use much more memory, and we incur the initial load time
		   at init.
		 */
		[self allocateTableViewCells];
	}
	return self;
}


- (void) allocateTableViewCells {
	/* Since myTableCells is a dynamic array object that we may have created before, we need to
	   release and recreate it.  If this is the first time, myTableCells is already nil so the release
	   has no effect. */
	
	[myTableCells release];
	myTableCells = [[NSMutableArray alloc] init];
	
	for (int i = 0; i < kPreallocTableViewController_NumCells; i++) {
		UITableViewCell *cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:nil] autorelease];
		cell.textLabel.text = [NSString stringWithFormat:@"This is the cell at index %d", i];
		[myTableCells addObject:cell];
	}
	
	/* We now have an array, myTableCells, with all of the table cells preallocated */
}


- (BOOL) shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation {
	return YES;
}


- (void)dealloc {
	[myTableCells release];
    [super dealloc];
}


/* -------------------------- TABLE DELEGATE METHODS ----------------------- */

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
	[[[[UIAlertView alloc] initWithTitle:@"Congrats" message:[NSString stringWithFormat:@"You pressed the cell at index %d", indexPath.row] delegate:nil cancelButtonTitle:@"I'm special" otherButtonTitles:nil] autorelease] show];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [myTableCells count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	return [myTableCells objectAtIndex:indexPath.row];
}


@end
